import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, DateRange, Form, LoadingService, Modal, ModalService, Row, DialogService, Select, SelectMultiple, TextInput, Time, Date as Data, Button,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata, ViewService,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

interface RowTarefas {
    CDCLIENTE: string;
    CDOPERADOR: string;
    CDOPERADORPAI: string;
    DTHORAAGEND: string;
    DTHRFINALAGEND: string;
    IDATIVO: string;
    IDOBRIGAOBSERVACAO: string;
    IDSITUACAOAGEND: string;
    IDTIPOMARCACAOTAREFA: string;
    IDTIPORECORRENCIA: string;
    NMCHECKLIST: string;
    NMCHECKLISTAGEND: string;
    NMFANTCLIE: string;
    NMLOCALCLIENTE: string;
    NMOPERADOR: string;
    NMRAZSOCCLIE: string;
    NMTIPOLOCALCLI: string;
    NRATIVIDADECHK: number;
    NRCHECKLIST: number;
    NRCHECKLISTAGEND: number;
    NRCONTRATOFCT: string;
    NRCRITERIOTAREFACHK: string;
    NRESTRUTURAM: number;
    NRLOCALCLIENTE: number;
    NRPRIORIDADETAREFA: number;
    NRSEQCONTRATO: number;
    NRTAREFACHK: string;
    NRTIPOLOCALCLI: number;
}

export class Fm00004000TarefasMassaController {
    private modalPreencherInformacoes!: Modal;

    private cadastroTarefaMassa!: Modal;

    private exibirInformacoesTarefaModal!: Modal;

    private modalConflito!: Modal;

    public modalConflitoEdicaoTarefa!: Modal;

    public visibilidadeModalConflitoEdicaoTarefa: boolean = false;

    public visibilidadeGridItens: boolean = false;

    public visibilidadeForm: boolean = true;

    public parametrizacao = '';

    public row: any = [];

    private basicModalCriterio!: Modal;

    public disabledPreencher: boolean = true;

    public disabledPeriodo: boolean = false;

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCRITERIOTAREFACHK: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSCRITERIOTAREFA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public onMountedTipoInsercao({ component }: IEventParam<Select>) {
        component.datasource.data = [];
        (component.datasource as MemoryDatasource).allData = [];

        const valores = [
            {
                name: 'Considerar todos os dias',
                value: 1,
            },
            {
                name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                value: 2,
            },
            {
                name: 'Considerar dias úteis (Não criar tarefa)',
                value: 3,
            },
        ];

        setTimeout(() => {
            if (this.parametrizacao === 'N') {
                const codARemover = 2;
                const index = valores.findIndex((item: any) => item.value === codARemover);

                if (index !== -1) {
                    valores.splice(index, 1);
                }
            } else {
                valores.splice(1, 0, {
                    name: 'Considerar dias úteis (Criar tarefa no próximo dia útil)',
                    value: 2,
                });
            }

            component.datasource.updateData(valores);
        }, 500);
    }

    public changeRecorrencia({ component }: IEventParam<Select>) {
        const data = Metadata.getInstance<Data>('select-data');
        const xDias = Metadata.getInstance<TextInput>('input-x-dias');
        const periodo = Metadata.getInstance<DateRange>('select-periodo');
        const dtFinal = Metadata.getInstance<Data>('select-data-limite');

        switch (component.value) {
            case 'SR':
                this.disabledPeriodo = false;
                data.removeValidation('required');
                data.disabled = true;
                data.value = null;
                periodo.addValidation('required', {});
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.removeValidation('required');
                dtFinal.disabled = true;
                break;
            case 'M':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'MP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'Q':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'QP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'S':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'SP':
                periodo.addValidation('required', {});
                this.disabledPeriodo = false;
                data.disabled = true;
                data.removeValidation('required');
                data.value = null;
                xDias.disabled = true;
                xDias.value = null;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            case 'P':
                data.addValidation('required', {});
                this.disabledPeriodo = true;
                data.disabled = false;
                periodo.removeValidation('required');
                periodo.value = null;
                xDias.disabled = false;
                dtFinal.value = null;
                dtFinal.disabled = false;
                dtFinal.addValidation('required', {});
                break;
            default:
                break;
        }
    }

    public preencherGrid() {
        const form = Metadata.getInstance<Form>('form-preencher-massa');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        const tipoMarcacao = Metadata.getInstance<Select>('IDTIPOMARCACAOTAREFA').value;
        const hora = Metadata.getInstance<Time>('selectHoraInicio').value;
        const duracao = Metadata.getInstance<TextInput>('selectDuracao').value;
        const operador = Metadata.getInstance<Select>('select-operador').value;
        const encarregado = Metadata.getInstance<Select>('ENCARREGADO').value;

        grid.columns[6].setLookupDataFilter({ CDOPERADOR: operador, NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value });
        if (Utilities.validateForm(form)) {
            grid.selectedRows.forEach((element: any) => {
                const newRow: any = {
                    ...element,
                    IDTIPOMARCACAOTAREFA: tipoMarcacao,
                    HORAINICIO: hora,
                    DURACAO: duracao,
                    OPERADOR: operador,
                    ENCARREGADO: encarregado,
                };
                grid.datasource.put(newRow);
            });
            grid.datasource.get();
            grid.selectedRows = [];
            this.fecharModal();
        }
    }

    public habilitarAcao({ component }: IEventParam<TekGrid>) {
        this.disabledPreencher = (component.selectedRows.length <= 0);
    }

    // TRANSFERÊNCIA AUTOMÁTICA DE OPERADORES
    private logModal!: Modal;

    public abrirModal() {
        this.modalPreencherInformacoes.show();
        this.validaParametrizacao();

        setTimeout(() => {
            Metadata.getInstance<Form>('form-preencher-massa').clearForm();
            //  trecho do modal de preenchimento em massa
            const hora = Metadata.getInstance<Time>('selectHoraInicio');
            const duracao = Metadata.getInstance<TextInput>('selectDuracao');
            const att = Metadata.getInstance<Row>('row-atencao');
            const operador = Metadata.getInstance<Select>('select-operador');
            const encarregado = Metadata.getInstance<Select>('ENCARREGADO');

            if (this.parametrizacao !== 'N') {
                hora.disabled = true;
                // hora.removeValidation('required');
                duracao.disabled = true;
                duracao.removeValidation('required');
                duracao.removeValidation('minLength');
                att.isVisible = true;
            } else {
                hora.disabled = false;
                hora.addValidation('required');
                duracao.disabled = false;
                duracao.addValidation('required');
                att.isVisible = false;
            }

            operador.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
            });

            if (Metadata.getInstance<Select>('selectCriarVistoria').value === '2') {
                encarregado.disabled = true;
                encarregado.removeValidation('required');
            } else {
                encarregado.disabled = false;
                encarregado.addValidation('required', {});
            }
        }, 500);
    }

    public changeEncarregadoPreencherMassa({ component }: IEventParam<Select>) {
        Metadata.getInstance<Select>('ENCARREGADO').datasource.setFilter({
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
            CDOPERADOR: component.value,
        });
    }

    public modConflito: boolean = false;

    public abrirModalConflito(parametros: any) {
        this.modalConflito.show();
        this.modConflito = true;
        setTimeout(() => {
            this.preencherGridConflitos(parametros);
        }, 500);
    }

    public destruirModal() {
        this.modalPreencherInformacoes?.destroy();
        this.modalConflito?.destroy();
        // TRANSFERÊNCIA AUTOMÁTICA DE OPERADORES
        this.logModal?.destroy();
    }

    public fecharModal() {
        this.modalPreencherInformacoes.hide();
    }

    public fecharModalConflito() {
        this.modalConflito.hide();
    }

    public habilitaOpcaoDias: boolean = true;

    public async exibirModalCadastroMassa({ event }: any) {
        event.preventDefault();
        this.cadastroTarefaMassa = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_005_modal_cadastro_tarefas_massa' });
        this.validaParametrizacao();
        this.habilitaOpcaoDias = (this.parametrizacao === 'N');
        this.cadastroTarefaMassa.show();
        this.modConflito = false;
    }

    public async exibirModalInformacoesTarefas({ row, event }: { row: RowTarefas, event: any }): Promise<void> {
        LoadingService.show();
        event.preventDefault();
        this.criarModal();
        this.exibirInformacoesTarefaModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_001_modal_tarefas' });
        this.exibirInformacoesTarefaModal.show();
        setTimeout(() => {
            this.popularModalInforamcoesTarefas(row);
        }, 2500);
        LoadingService.hide();
    }

    public async criarModal() {
        this.modalPreencherInformacoes = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_006_modal_preencher_massa', true);
        this.modalConflito = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_004_modal_conflito_datas', true);
        this.modalConflitoEdicaoTarefa = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_00004_004_modal_conflito_datas_modal_edicao', true);
        this.logModal = await ModalService.createFromJson('fm_00004_000_tarefas/modal/fm_modal_log_transferencia', true);
    }

    public async fecharModalTarefasMassa() {
        this.cadastroTarefaMassa.hide();
        this.cadastroTarefaMassa?.destroy();
    }

    public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(`formTarefas_${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    /* INTEGRAÇÃO COM SM */
    public async changeContratos({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (grid.editing) {
            await Metadata.getInstance<Select>(component.name).datasource.setFilter({
                NRESTRUTURAM: grid.datasource.currentRow.NRESTRUTURAM,
            });
            grid.columns[1].setLookupDataFilter({ NRESTRUTURAM: grid.datasource.currentRow.NRESTRUTURAM });
            this.focusContratos({ component });
        }
    }

    public focusContratos({ component }: IEventParam<Select>) {
        if (component.datasource.data.length > 0) {
            component.addValidation('required', {});
        } else {
            component.removeValidation('required');
        }
    }

    public async validationsContrato({ component }:IEventParam<TekGrid>) {
        ViewService.nextTick(() => {
            component.datasource.data.forEach(async (element: any) => {
                const comp: any = component.getComponent(element.NRLOCALCLIENTE, 'NRSEQCONTRATO');
                await comp.datasource.setFilter({
                    NRESTRUTURAM: element.NRESTRUTURAM,
                });

                if (comp.datasource.data.length > 0) {
                    comp.addValidation('required', {});
                } else {
                    comp.removeValidation('required');
                }
            });
        });
    }

    /* INTEGRAÇÃO COM SM */

    public operador: any = '';

    public focusOperadores({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (grid.editing) {
            component.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
                NRTAREFACHK: Metadata.getInstance<SelectMultiple>('NRTAREFACHK').value,
            });
        }
    }

    public changeEncarregado({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        this.operador = component.value;
        grid.columns[6].setLookupDataFilter({ CDOPERADOR: component.value, NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value });
    }

    public focusEncarregado({ component }: any) {
        const grid = (this.modConflito) ? Metadata.getInstance<TekGrid>('tek-grid-conflito-datas') : Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        if (grid.editing) {
            component.datasource.setFilter({
                CDOPERADOR: this.operador,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
            });
        }
    }

    public validaVistoriaGrid({ component }: any) {
        // const vistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        const observacao = Metadata.getInstance<Select>('selectObservacoesObrigatorias');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (component.value === '1') {
            observacao!.datasource!.data!.push({
                value: '3',
                text: 'Somente na vistoria',
            });
            observacao!.datasource!.data!.push({
                value: '4',
                text: 'Na tarefa e na vistoria',
            });
            grid.columns[6].editable = true;
            // component.disabled = false;
            // component.addValidation('required');
        } else {
            grid.columns[6].editable = false;
            observacao.datasource?.get();
            // component.disabled = true;
            // component.removeValidation('required');
        }
    }

    public changeTipoDeLocal({ component }: IEventParam<Select>) {
        const tipoLocal = Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI');
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        tipoLocal.value = null;
        local.value = null;
        checklist.value = null;

        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            local.disabled = true;
        }
    }

    public changeLocal({ component }: IEventParam<SelectMultiple>) {
        const local = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        if (component.value.length > 0) {
            local.disabled = false;
            checklist.disabled = false;

            local.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'N',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
            });

            checklist.datasource.setFilter({
                NRTIPOLOCALCLI: component.value,
                UNICA: 'N',
            });
        } else {
            local.value = null;
            checklist.value = null;
            local.disabled = true;
            checklist.disabled = true;
        }
    }

    public changeItensChecklist({ component }: IEventParam<Select>) {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK');
        if (component.value) {
            itenschecklist.disabled = false;
            itenschecklist.datasource.setFilter({
                NRCHECKLIST: component.value,
            });
        } else {
            itenschecklist.value = null;
            itenschecklist.disabled = true;
        }
    }

    public changeEscopo({ component }: IEventParam<SelectMultiple>) {
        const escopo = Metadata.getInstance<Select>('NRESCOPOSTAREFA');
        if (component.value) {
            escopo.disabled = false;
        } else {
            escopo.value = null;
            escopo.disabled = true;
        }
    }

    public changeTarefa({ component }: IEventParam<SelectMultiple>) {
        const tarefa = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');

        if (component.value) {
            tarefa.disabled = false;
            tarefa.datasource.setFilter({
                NRATIVIDADECHK: itenschecklist.value,
            });
        } else {
            tarefa.value = null;
            tarefa.disabled = true;
        }
    }

    public get itensCheckList(): object {
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        return itenschecklist.value;
    }

    public get itensCheckListRow(): object {
        const itenschecklist = Metadata.getInstance<Select>('informacoesItensChecklist');
        return itenschecklist.value;
    }

    public setCriterios({ component }: IEventParam<SelectMultiple>) {
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const gridCriterios = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        const prioridade = Metadata.getInstance<Select>('select-prioridade');

        if (component.value) {
            gridCriterios.datasource?.setFilter({
                NRTAREFACHK: component.value,
                NRATIVIDADECHK: itenschecklist.value,
                NRCHECKLIST: checklist.value,
            });
            gridCriterios.datasource?.get();
            gridCriterios.isVisible = true;

            // trecho de código adicionado para validação de prioridade
            const arrayPrioritario: any = component.selectValue.find((array: any) => array.IDPRIORIDADE === 1);
            if (arrayPrioritario) {
                prioridade.disabled = true;
                prioridade.value = arrayPrioritario.NRPRIORIDADETAREFA;
            } else {
                prioridade.disabled = false;
                prioridade.value = null;
            }
        } else {
            component.value = null;
            gridCriterios.isVisible = false;
        }
    }

    public executorHora() {
        const dados: any = [];
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        grid.datasource.data.forEach((element: any) => {
            dados.push({
                HORAINICIO: element.HORAINICIO,
                DURACAO: element.DURACAO,
                FUNCIONARIO: element.OPERADOR,
                ENCARREGADO: element.ENCARREGADO,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRSEQCONTRATO: element.NRSEQCONTRATO,
                IDTIPOMARCACAOTAREFA: element.IDTIPOMARCACAOTAREFA,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
                IGNORARTAREFA: 'N',
            });
        });
        return dados;
    }

    public validarFormEGrid() {
        // const formTarefas = Metadata.getInstance<Form>('formCadastroTarefas');
        // const formCriterios = Metadata.getInstance<Form>('formTarefas_dropBoxCriterioAvaliacao');
        // const formAgendamento = Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento');
        // const tekGridCriteriosDeAvaliacaoCadastro = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        // const diaSemana = Metadata.getInstance<SelectMultiple>('selectDiaSemana').value;
        // const dataRange = Metadata.getInstance<DateRange>('select-periodo').value;
        // const data = Metadata.getInstance<DateRange>('select-data').value;
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        if (grid.isGridValid(true)) {
            // Utilities.showSuccessMessage();
            const gridteste = JSON.stringify(grid.datasource);
            Utilities.alertMessage(gridteste, 'error');
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
        // this.inserirTarefa();
    }

    public inserirTarefa() {
        const formTarefas = Metadata.getInstance<Form>('formCadastroTarefas');
        const formCriterios = Metadata.getInstance<Form>('formTarefas_dropBoxCriterioAvaliacao');
        const formAgendamento = Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento');
        const tekGridCriteriosDeAvaliacaoCadastro = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        const diaSemana = Metadata.getInstance<SelectMultiple>('selectDiaSemana').value;
        const dataRange = Metadata.getInstance<DateRange>('select-periodo').value;
        const data = Metadata.getInstance<DateRange>('select-data').value;
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');

        if (Utilities.validateGrid(grid)) {
            if (Utilities.validateForm(formTarefas) && Utilities.validateForm(formCriterios) && Utilities.validateForm(formAgendamento)) {
                const parametros = {
                    NOMETAREFA: Metadata.getInstance<TextInput>('nomeTarefa').value,
                    CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
                    NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI').value,
                    NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                    NRCHECKLIST: Metadata.getInstance<Select>('NRCHECKLIST').value,
                    NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK').value,
                    NRESCOPOSTAREFA: Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA').value,
                    NRTAREFACHK: Metadata.getInstance<SelectMultiple>('NRTAREFACHK').value,
                    NRCRITERIOTAREFACHK: tekGridCriteriosDeAvaliacaoCadastro.selectedRows.map((element: any) => element),
                    DATERANGE: (dataRange.length === 0) ? [data, data] : dataRange,
                    DIASEMANA: diaSemana,
                    VISTORIA: Metadata.getInstance<Select>('selectCriarVistoria').value,
                    OBSERVACOES: Metadata.getInstance<Select>('selectObservacoesObrigatorias').value,
                    EXECUTOR: this.executorHora(),
                    RECORRENCIA: Metadata.getInstance<Select>('select-recorrencia').value,
                    QTDDIAS: Metadata.getInstance<TextInput>('input-x-dias').value,
                    DTTERMINO: Metadata.getInstance<Data>('select-data-limite').value,
                    NRPRIORIDADETAREFA: Metadata.getInstance<Select>('select-prioridade').value,
                };
                this.salvarTarefa(parametros, this.executorHora(), false);
            }
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }

    public salvarTarefa(parametros: any, executor: any, conflito: boolean) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tarefas');

        LoadingService.show();
        Http.post('/insertTarefasRecorrentes', {
            NOMETAREFA: parametros.NOMETAREFA,
            CDCLIENTE: parametros.CDCLIENTE,
            NRTIPOLOCALCLI: parametros.NRTIPOLOCALCLI,
            NRLOCALCLIENTE: parametros.NRLOCALCLIENTE,
            NRCHECKLIST: parametros.NRCHECKLIST,
            NRATIVIDADECHK: parametros.NRATIVIDADECHK,
            NRESCOPOSTAREFA: parametros.NRESCOPOSTAREFA,
            NRTAREFACHK: parametros.NRTAREFACHK,
            NRCRITERIOTAREFACHK: parametros.NRCRITERIOTAREFACHK,
            DATERANGE: parametros.DATERANGE,
            DIASEMANA: parametros.DIASEMANA,
            VISTORIA: parametros.VISTORIA,
            OBSERVACOES: parametros.OBSERVACOES,
            EXECUTOR: executor,
            RECORRENCIA: (conflito === true) ? 'SR' : parametros.RECORRENCIA,
            QTDDIAS: parametros.QTDDIAS,
            DTTERMINO: parametros.DTTERMINO,
            CONFLITO: conflito,
            PARAMETRIZACAO: this.parametrizacao,
            NRPRIORIDADETAREFA: parametros.NRPRIORIDADETAREFA,
        }).then((response: IDictionary) => {
            if (response.data !== 'OK') {
                this.abrirModalConflito(response.data);
                Utilities.alertMessage('Ainda há conflitos existentes. Resolva-os para prosseguir.', 'error');
            } else {
                Utilities.showSuccessMessage();
                this.fecharModalConflito();
                this.fecharModalTarefasMassa();
                grid.datasource.get();
            }
        }).catch((error: any) => {
            // console.error(error);
            if (error && error.response && error.response.data && error.response.data.error && error.response.data.error.includes('Undefined index:')) {
                let [erroCampo] = error.response.data.error.split('\'').filter((elemento: any) => elemento.includes('Undefined index:'));
                erroCampo = erroCampo ? erroCampo.replaceAll('\'', '').replace('Undefined index: ', '') : erroCampo;
                if (!erroCampo) {
                    Utilities.showErrorMessage(error);
                } else {
                    Utilities.showDialog('Erro', 'error', 'Verifique se todos os campos obrigatórios foram preenchidos corretamente.');
                }
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dadosBanco: any = {};

    public preencherGridConflitos(parametros: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-datas');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        /* eslint-disable */
        let cont = 0;

        for (const key in parametros.DADOSCONFLITO) {
            const dados: any = {
                CHAVE: cont,
                NRLOCALCLIENTE: parametros.DADOSCONFLITO[key].NRLOCALCLIENTE,
                DATAINICIAL: parametros.DADOSCONFLITO[key].DATAINICIAL,
                DATAFINAL: parametros.DADOSCONFLITO[key].DATAFINAL,
                IDTIPOMARCACAOTAREFA: parametros.DADOSCONFLITO[key].IDTIPOMARCACAOTAREFA,
                HORAINICIO: parametros.DADOSCONFLITO[key].HORAINICIO,
                DURACAO: parametros.DADOSCONFLITO[key].DURACAO,
                OPERADOR: parametros.DADOSCONFLITO[key].FUNCIONARIO,
                ENCARREGADO: parametros.DADOSCONFLITO[key].ENCARREGADO,
                IGNORARTAREFA: 'N',
            };
            grid.datasource.post(dados);
            cont++;
        }

        grid.datasource.get();
        grid.columns[6].setLookupDataFilter({ NRLOCALCLIENTE: parametros.NRLOCALCLIENTE });
        grid.columns[7].setLookupDataFilter({ NRLOCALCLIENTE: parametros.NRLOCALCLIENTE });

        /* eslint-enable */
        this.dadosBanco = parametros;
    }

    public editedRows: any = '';

    public salvarConflitos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-conflito-datas');
        const dados: any = [];

        if (this.editedRows.length !== 0) {
            grid.datasource.data.forEach((element: any) => {
                dados.push({
                    HORAINICIO: element.HORAINICIO,
                    DURACAO: element.DURACAO,
                    FUNCIONARIO: element.OPERADOR,
                    ENCARREGADO: element.ENCARREGADO,
                    NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                    NRSEQCONTRATO: element.NRSEQCONTRATO,
                    IDTIPOMARCACAOTAREFA: element.IDTIPOMARCACAOTAREFA,
                    DATAINICIAL: element.DATAINICIAL.split(' ')[0],
                    DATAFINAL: element.DATAFINAL.split(' ')[0],
                    IGNORARTAREFA: element.IGNORARTAREFA,
                });
            });
            const parametros = {
                NOMETAREFA: this.dadosBanco.NOMETAREFA,
                CDCLIENTE: this.dadosBanco.CDCLIENTE,
                NRTIPOLOCALCLI: this.dadosBanco.NRTIPOLOCALCLI,
                NRLOCALCLIENTE: this.dadosBanco.NRLOCALCLIENTE,
                NRCHECKLIST: this.dadosBanco.NRCHECKLIST,
                NRATIVIDADECHK: this.dadosBanco.NRATIVIDADECHK,
                NRESCOPOSTAREFA: this.dadosBanco.NRESCOPOSTAREFA,
                NRTAREFACHK: this.dadosBanco.NRTAREFACHK,
                NRCRITERIOTAREFACHK: this.dadosBanco.NRCRITERIOTAREFACHK,
                DATERANGE: this.dadosBanco.DATERANGE,
                DIASEMANA: this.dadosBanco.DIASEMANA,
                VISTORIA: this.dadosBanco.VISTORIA,
                OBSERVACOES: this.dadosBanco.OBSERVACOES,
                QTDDIAS: this.dadosBanco.QTDDIAS,
                DTTERMINO: this.dadosBanco.DTTERMINO,
            };
            this.salvarTarefa(parametros, dados, true);
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }

    public validarGrid({ component }: IEventParam<TekGrid>) {
        if (component.isGridValid(true)) {
            Utilities.showSuccessMessage();
            this.editedRows = component.getEditedRows();
        } else {
            Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
        }
    }

    public validaParametrizacao() {
        Http.get('getParametroConflito').then((response) => {
            this.parametrizacao = (response.data.length !== 0) ? response.data[0].IDVALIDACRIATAREFA : null;
        });
    }

    public defineTarefa() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        const locais = Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').selectValue;
        locais.forEach((element: any) => {
            const dados: any = {
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRESTRUTURAM: element.NRESTRUTURAM,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
        grid.columns[5].setLookupDataFilter({ NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value });
    }

    // Rotina log de transferência automática de operadores
    public async abrirModalLog({ component, event }: IEventParam<Modal>) {
        event?.stopPropagation();
        await this.logModal.show();
        Metadata.getInstance<TekGrid>('grid-log-tarefas').datasource.setFilter({
            NRCHECKLISTAGEND: component.name.split('_')[1],
        });
    }

    public fecharModalLog() {
        this.logModal.hide();
    }

    public excluirTarefas({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00004000TarefasMassaController.aposConfirmarDelete', 'Excluir Registro?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        LoadingService.show();
        Http.post('/inativarTarefas', {
            NRS: component.selectedRows.map((element: any) => element.NRCHECKLISTAGEND),
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public setFiltroGridPrincipal({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    private popularModalInforamcoesTarefas(row: RowTarefas): void {
        Metadata.getInstance<TextInput>('informacoesNomeTarefa').value = row.NMCHECKLISTAGEND;
        this.setValueSelectCamposModal<Select>('informacoesNomeCliente', row.CDCLIENTE);
        this.changeCampoChecklist(row);
        this.changeCampoTipoDeLocal(row);
        this.changeCampoLocalModalInformacoes(row);
        this.changeItensChecklistModalInformacoes(row);
        this.changeEscopoModalInformacoes(row);
        this.changeAtividadeModalInformacoes(row);
        this.changeContratosModalInformacoes(row);
        this.changeFuncionarioModalInformacoes(row);
        this.changeEncarregadoModalInformacoes(row);
        this.setDataTarefaModalInformacoes(row);
        this.setHorarioTarefaModalInformacoes(row);
        this.setDuracaodaTarefaModalInformacoes(row);
        this.setObervacoesObrigatorias(row);
        this.changeCriterioDeAtividadesModalInformacoes(row);
        this.setValueSelectCamposModal<Select>('select-Informacoes-prioridade', row.NRPRIORIDADETAREFA);
        this.setValueSelectCamposModal<Select>('selectInformacoesOpcoesMarcacao', row.IDTIPOMARCACAOTAREFA);
    }

    private changeCriterioDeAtividadesModalInformacoes(row: RowTarefas): void {
        const arrayNrtarefachkString = row.NRTAREFACHK.split(', ').map(String);
        const grid = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-edicao');
        grid.datasource?.setFilter({
            NRTAREFACHK: arrayNrtarefachkString,
            NRATIVIDADECHK: [row.NRATIVIDADECHK],
            NRCHECKLIST: row.NRCHECKLIST,
        });
    }

    private setDuracaodaTarefaModalInformacoes(row: RowTarefas): void {
        const horarioInicialString: string = row.DTHORAAGEND.slice(11, 16);
        const horarioFinalString: string = row.DTHRFINALAGEND.slice(11, 16);

        const horarioInicial: Date = new Date(`1970-01-01T${horarioInicialString}:00`);
        const horarioFinal: Date = new Date(`1970-01-01T${horarioFinalString}:00`);

        const diferencaMinutos: number = (horarioFinal.getTime() - horarioInicial.getTime()) / (1000 * 60);

        const duracaoTarefa = diferencaMinutos.toString();

        Metadata.getInstance<Time>('selectInformacoesdDuracao1').value = (`00:${duracaoTarefa}`);
    }

    private setDataTarefaModalInformacoes(row: RowTarefas): void {
        const data = row.DTHORAAGEND.slice(0, 10);
        Metadata.getInstance<Data>('Informacoes-DTHORAAGEND').value = data;
    }

    private setHorarioTarefaModalInformacoes(row: RowTarefas): void {
        const horario = row.DTHORAAGEND.slice(11, 16);
        Metadata.getInstance<Time>('selectInformacoesHoraInicio1').value = horario;
    }

    private setValueSelectCamposModal<T extends Select | SelectMultiple>(field: string, props: T | T[] | string | number | number[]): void {
        Metadata.getInstance<T>(field).setValue(props, false);
    }

    private changeEncarregadoModalInformacoes(row: RowTarefas): void {
        const encarregado = Metadata.getInstance<Select>('selectInformacoesEncarregado1');
        encarregado.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
            CDOPERADOR: row.CDOPERADOR,
        });

        this.setValueSelectCamposModal<Select>(encarregado.name, row.CDOPERADORPAI);
    }

    private changeFuncionarioModalInformacoes(row: RowTarefas): void {
        const funcionario = Metadata.getInstance<Select>('selectInformacoesFuncionario1');
        funcionario.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(funcionario.name, row.CDOPERADOR);
    }

    private changeContratosModalInformacoes(row: RowTarefas): void {
        const selectContratos = Metadata.getInstance<Select>('informacoes-NRSEQCONTRATO');
        selectContratos.datasource.setFilter({
            NRESTRUTURAM: row.NRESTRUTURAM,
        });
        this.setValueSelectCamposModal<Select>(selectContratos.name, row.NRSEQCONTRATO);
    }

    private changeAtividadeModalInformacoes(row: RowTarefas): void {
        const atividade = Metadata.getInstance<SelectMultiple>('informacoesAtividadeTarefa');
        atividade.datasource.setFilter({
            NRATIVIDADECHK: [row.NRATIVIDADECHK],
        });
        const arrayNrtarefachk: number[] = row.NRTAREFACHK.split(', ').map(Number);
        this.setValueSelectCamposModal<SelectMultiple>(atividade.name, arrayNrtarefachk);
    }

    private changeEscopoModalInformacoes(row: RowTarefas): void {
        const escopo = Metadata.getInstance<SelectMultiple>('informacoesEscopoTarefa');
        escopo.datasource.setFilter({
            NRTAREFACHK: row.NRATIVIDADECHK,
        });
        this.setValueSelectCamposModal<SelectMultiple>(escopo.name, [row.NRATIVIDADECHK]);
    }

    private changeItensChecklistModalInformacoes(row: RowTarefas): void {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('informacoesItensChecklist');
        itenschecklist.datasource.setFilter({
            NRCHECKLIST: row.NRCHECKLIST,
        });
        this.setValueSelectCamposModal<SelectMultiple>(itenschecklist.name, [row.NRATIVIDADECHK]);
    }

    private changeCampoLocalModalInformacoes(row: RowTarefas): void {
        const local = Metadata.getInstance<Select>('informacoesLocalCli');
        local.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(local.name, row.NRLOCALCLIENTE);
    }

    private changeCampoChecklist(row: RowTarefas): void {
        const checklist = Metadata.getInstance<Select>('informacoesNRCHECKLIST');
        checklist.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
        });
        this.setValueSelectCamposModal<Select>(checklist.name, row.NRCHECKLIST);
    }

    private setObervacoesObrigatorias(row: RowTarefas): void {
        const observacoes = Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias');

        if (row.IDOBRIGAOBSERVACAO === 'N') {
            this.setValueSelectCamposModal<Select>(observacoes.name, '1');
        } else {
            this.setValueSelectCamposModal<Select>(observacoes.name, '2');
        }
    }

    private changeCampoTipoDeLocal(row: RowTarefas): void {
        const tipoLocal = Metadata.getInstance<Select>('informacoesTipoLocalCli');
        tipoLocal.datasource.setFilter({
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(tipoLocal.name, row.NRTIPOLOCALCLI);
    }

    public rowClickFiltroGrid() {
        this.visibilidadeGridItens = true;
        this.visibilidadeForm = !this.visibilidadeGridItens;
    }

    public async fecharModalInformacoesTarefas() {
        this.exibirInformacoesTarefaModal.hide();
        this.exibirInformacoesTarefaModal?.destroy();
    }

    public async setFilterEncarregado({ component }: IEventParam<Select>) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE').value;
        const encarregado = Metadata.getInstance<Select>('ENCARREGADO');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        if (component.value) {
            encarregado.disabled = false;

            await encarregado.datasource.setFilter({
                NRLOCALCLIENTE: local,
                CDOPERADOR: component.value,
            });

            setTimeout(() => {
                if (encarregado.datasource.data.length <= 0 && criaVistoria.value === '1') {
                    const message = `O operador ${component.selectValue.NMOPERADOR} é o usuário máximo da Hierarquia atual. Gostaria de proceder com a criação de uma vistoria referente a esta tarefa para ele?`;
                    this.dialogConfirmarCriacaoVistoria(message);
                }
            });
        } else {
            encarregado.disabled = true;
        }
    }

    public dialogConfirmarCriacaoVistoria(msg: string) {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Atenção!!',
            text: msg,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: '{{Fm00004000TarefasMassaController.acaoDialog}}',
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: '{{Fm00004000TarefasMassaController.acaoDialog}}',
                    },
                },
            ],
        });
    }

    public acaoDialog({ component }: IEventParam<Button>) {
        const encarregado = Metadata.getInstance<Select>('ENCARREGADO');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');

        // Significa que o usuário não deseja continuar com a criação da vistoria para um Operador Máximo na Hierarquia
        if (component.label === 'Não') {
            criaVistoria.value = '2';
        }

        encarregado.removeValidation('required');
        encarregado.disabled = true;
        DialogService.hide();
    }

    public populaEscopo() {
        const escopoServico = Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA');
        Http.post('/populaEscopo', { NRTAREFACHK: Metadata.getInstance<Select>('NRTAREFACHK').value }).then((response: any) => {
            escopoServico.value = response.data.data;
        });
    }

    public populaCriterios() {
        const tarefa = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const checkbox = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        if (tarefa.value.length > 0) {
            checkbox.datasource?.setFilter({
                NRTAREFACHK: tarefa.value,
                NRATIVIDADECHK: itenschecklist.value,
                NRCHECKLIST: checklist.value,
            });
            checkbox.datasource?.get();
            checkbox.isVisible = true;
            Http.post('/populaCriterios', { NRTAREFACHK: Metadata.getInstance<Select>('NRTAREFACHK').value }).then((response: any) => {
                checkbox.value = response.data;
            });
        } else {
            tarefa.setValue([]);
            checkbox.isVisible = false;
        }
    }

    public formatarDataHora(dtinicio: String, dtfim: String) {
        const [horaini, minutoini] = dtinicio[1].split(':');
        const [horafim, minutofim] = dtfim[1].split(':');
        const hora = (parseInt(horafim, 10) - parseInt(horaini, 10)).toString().padStart(2, '0');
        const minuto = (parseInt(minutofim, 10) - parseInt(minutoini, 10)).toString().padStart(2, '0');
        return `${hora}:${minuto}`;
    }

    public popularCriteriosModal(nrChecklist: Number, nrAtividadechk: Number, nrTarefachk: Number) {
        const checkbox = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        checkbox.datasource?.setFilter({
            NRTAREFACHK: nrTarefachk,
            NRATIVIDADECHK: nrAtividadechk,
            NRCHECKLIST: nrChecklist,
            MODAL: true,
        });
        checkbox.datasource?.get();
        checkbox.isVisible = true;
    }

    public async exibirModalCriterios() {
        this.basicModalCriterio = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_002_modal_cadastro_criterio' });
        this.basicModalCriterio.show();
    }

    public async fecharModalCriterio() {
        this.basicModalCriterio.hide();
        this.basicModalCriterio?.destroy();
    }

    public novoCriterio() {
        const gridCriterios = Metadata.getInstance<TekGrid>('tek-grid-criterios-de-avaliacao-cadastro');
        const editedRows = gridCriterios.getEditedRows();
        LoadingService.show();
        Http.post('/salvarNovoCriterioTekGrid', {
            ATIVIDADES: editedRows[0].NRTAREFACHK,
            NOME: editedRows[0].DSCRITERIOTAREFA,
        }).then(() => {
            Utilities.showSuccessMessage();
            gridCriterios.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
