import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, TextInput,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class FmtParametrizacaoCategManut {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCATEGORIAMANUT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCATEGORIAMANUT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public saveCategoriaManutencao({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('saveCategoriaManutencao', {
                ROWS: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtParametrizacaoCategManut.deleteCategoriasManutencao', 'Excluir Categorias de Manutenção');
    }

    public deleteCategoriasManutencao() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-categoria-manutencao');

        LoadingService.show();
        Http.post('/deleteCategoriasManutencao', {
            NRS: component.selectedRows.map((element: any) => element.NRCATEGORIAMANUT),
        }).then(() => {
            Utilities.showSuccessMessage();
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
