import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class FmtParametrizacaoPrioridades {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRPRIORIDADEMANUT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMPRIORIDADEMANUT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public savePrioridade({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('savePrioridade', {
                ROWS: component.getEditedRows(),
            }).then(() => {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtParametrizacaoPrioridades.deletePrioridades', 'Excluir Prioridades');
    }

    public deletePrioridades() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-prioridades');

        LoadingService.show();
        Http.post('/deletePrioridades', {
            NRS: component.selectedRows.map((element: any) => element.NRPRIORIDADEMANUT),
        }).then(() => {
            Utilities.showSuccessMessage();
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public getRowColor({ row }: IDictionary<any>) {
        return {
            'background-color': row.DSCORPRIORIDADE,
            color: row.DSCORPRIORIDADE,
            width: '5vw',
        };
    }

    public cancelarEdicao({ component }: any) {
        setTimeout(() => {
            component.cancelEditedRows();
        }, 4300);
    }
}
